
namespace RoboidControl {

    /// @brief An Incremental Encoder measures the rotations of an axle using a rotary
    /// sensor. Some encoders are able to detect direction, while others can not.
    public class RelativeEncoder : Thing {
        /// @brief Creates a sensor which measures distance from pulses
        /// @param pulsesPerRevolution The number of pulse edges which make a
        /// full rotation
        /// @param distancePerRevolution The distance a wheel travels per full
        /// rotation
        /*
        public RelativeEncoder(bool invokeEvent = true) : base(Type.IncrementalEncoder, invokeEvent) { }
        */
        public RelativeEncoder(Thing parent = default) : base(parent) {
            this.type = Type.RelativeEncoder;
        }

        protected float _rotationSpeed = 0;
        /// @brief Get the rotation speed since the previous call
        /// @param currentTimeMs The clock time in milliseconds
        /// @return The speed in rotations per second        
        public virtual float angularSpeed {
            get => _rotationSpeed;
            set {
                if (_rotationSpeed != value) {
                    _rotationSpeed = value;
                    this.owner.Send(new BinaryMsg(this));
                }
            }
        }

        public override byte[] GenerateBinary() {
            byte[] data = new byte[1];
            byte ix = 0;
            data[ix++] = (byte)(this._rotationSpeed * 127);
            return data;
        }
        public override void ProcessBinary(byte[] data) {
            this._rotationSpeed = (float)data[0] / 127;
        }

    };

} // namespace RoboidControl
